'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

// Looks for JWT token, and if it is found, sets some variables.

var authenticate = function () {
	var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(_ref2) {
		var _this = this;

		var authentication = _ref2.authentication;
		var keys = _ref2.keys;
		var validate_token = _ref2.validate_token;

		var _get_jwt_token, token, error, payload, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, secret, jwt_id, user_id, is_valid;

		return _regenerator2.default.wrap(function _callee$(_context) {
			while (1) {
				switch (_context.prev = _context.next) {
					case 0:
						// Get JWT token from incoming HTTP request
						_get_jwt_token = get_jwt_token(this);
						token = _get_jwt_token.token;
						error = _get_jwt_token.error;

						// Little helpers which can be called from routes to ensure
						// a logged in user or a specific user role

						this.authenticate = function () {
							throw new _errors2.default.Unauthenticated();
						};
						this.role = function () {
							throw new _errors2.default.Unauthenticated();
						};

						// If no JWT token was found, then done

						if (token) {
							_context.next = 8;
							break;
						}

						this.authentication_error = new _errors2.default.Unauthenticated(error);
						return _context.abrupt('return');

					case 8:

						// JWT token (is now accessible from Koa's `ctx`)
						this.jwt = token;

						// Verify JWT token integrity
						// by checking its signature using the supplied `keys`

						payload = void 0;
						_iteratorNormalCompletion = true;
						_didIteratorError = false;
						_iteratorError = undefined;
						_context.prev = 13;
						_iterator = (0, _getIterator3.default)(keys);

					case 15:
						if (_iteratorNormalCompletion = (_step = _iterator.next()).done) {
							_context.next = 33;
							break;
						}

						secret = _step.value;
						_context.prev = 17;

						payload = _jsonwebtoken2.default.verify(token, secret);
						return _context.abrupt('break', 33);

					case 22:
						_context.prev = 22;
						_context.t0 = _context['catch'](17);

						if (!(_context.t0.name === 'TokenExpiredError')) {
							_context.next = 27;
							break;
						}

						this.authentication_error = new _errors2.default.Unauthenticated('Token expired');
						return _context.abrupt('return');

					case 27:
						if (!(_context.t0.name === 'JsonWebTokenError')) {
							_context.next = 29;
							break;
						}

						return _context.abrupt('continue', 30);

					case 29:
						throw _context.t0;

					case 30:
						_iteratorNormalCompletion = true;
						_context.next = 15;
						break;

					case 33:
						_context.next = 39;
						break;

					case 35:
						_context.prev = 35;
						_context.t1 = _context['catch'](13);
						_didIteratorError = true;
						_iteratorError = _context.t1;

					case 39:
						_context.prev = 39;
						_context.prev = 40;

						if (!_iteratorNormalCompletion && _iterator.return) {
							_iterator.return();
						}

					case 42:
						_context.prev = 42;

						if (!_didIteratorError) {
							_context.next = 45;
							break;
						}

						throw _iteratorError;

					case 45:
						return _context.finish(42);

					case 46:
						return _context.finish(39);

					case 47:
						if (payload) {
							_context.next = 50;
							break;
						}

						this.authentication_error = new _errors2.default.Unauthenticated('Corrupt token');
						return _context.abrupt('return');

					case 50:

						// JWT token ID
						jwt_id = payload.jti;

						// `subject`
						// (which is a user id)

						user_id = payload.sub;

						// Optional JWT token validation (by id)
						// (for example, that it has not been revoked)

						if (!validate_token) {
							_context.next = 59;
							break;
						}

						_context.next = 55;
						return validate_token(token, this);

					case 55:
						is_valid = _context.sent.valid;

						if (is_valid) {
							_context.next = 59;
							break;
						}

						this.authentication_error = new _errors2.default.Unauthenticated('Token revoked');
						return _context.abrupt('return');

					case 59:

						// JWT token ID is now accessible via Koa's `ctx`
						this.jwt_id = jwt_id;

						// Extracts user data (description) from JWT token payload
						// (`user` is now accessible via Koa's `ctx`)
						this.user = authentication ? authentication(payload) : {};

						// Sets user id
						this.user.id = user_id;

						// Extra payload fields:
						//
						// 'iss' // Issuer
						// 'sub' // Subject
						// 'aud' // Audience
						// 'exp' // Expiration time
						// 'nbf' // Not before
						// 'iat' // Issued at 
						// 'jti' // JWT ID

						// JWT token payload is accessible via Koa's `ctx`
						this.token_data = payload;

						// The user is assumed authenticated now,
						// so `this.authenticate()` helper won't throw an exception.
						// (and will return the `user`)
						this.authenticate = function () {
							return _this.user;
						};

						// A little helper which can be called from routes
						// as `this.role('administrator')`
						// which will throw if the user isn't administrator
						// (`authentication` function needs to get 
						//  `role` from JWT payload for this to work)
						this.role = function () {
							for (var _len = arguments.length, roles = Array(_len), _key = 0; _key < _len; _key++) {
								roles[_key] = arguments[_key];
							}

							var _iteratorNormalCompletion2 = true;
							var _didIteratorError2 = false;
							var _iteratorError2 = undefined;

							try {
								for (var _iterator2 = (0, _getIterator3.default)(roles), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
									var role = _step2.value;

									if (_this.user.role === role) {
										return true;
									}
								}
							} catch (err) {
								_didIteratorError2 = true;
								_iteratorError2 = err;
							} finally {
								try {
									if (!_iteratorNormalCompletion2 && _iterator2.return) {
										_iterator2.return();
									}
								} finally {
									if (_didIteratorError2) {
										throw _iteratorError2;
									}
								}
							}

							throw new _errors2.default.Unauthorized('One of the following roles is required: ' + roles);
						};

					case 65:
					case 'end':
						return _context.stop();
				}
			}
		}, _callee, this, [[13, 35, 39, 47], [17, 22], [40,, 42, 46]]);
	}));

	return function authenticate(_x) {
		return _ref.apply(this, arguments);
	};
}();

// Koa middleware creator


exports.default = function (options) {
	// Koa middleware
	return function () {
		var _ref3 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2(ctx, next) {
			return _regenerator2.default.wrap(function _callee2$(_context2) {
				while (1) {
					switch (_context2.prev = _context2.next) {
						case 0:
							_context2.next = 2;
							return authenticate.call(ctx, options);

						case 2:
							_context2.next = 4;
							return next();

						case 4:
						case 'end':
							return _context2.stop();
					}
				}
			}, _callee2, this);
		}));

		return function (_x2, _x3) {
			return _ref3.apply(this, arguments);
		};
	}();
};

exports.issue_jwt_token = issue_jwt_token;

var _jsonwebtoken = require('jsonwebtoken');

var _jsonwebtoken2 = _interopRequireDefault(_jsonwebtoken);

var _errors = require('../errors');

var _errors2 = _interopRequireDefault(_errors);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// First looks for JWT token inside HTTP Authorization header,
// then looks for `authentication` cookie.
function get_jwt_token(context) {
	// Parses "Authorization: Bearer ${token}"
	if (context.header.authorization) {
		var match = context.header.authorization.match(/^Bearer (.+)$/i);

		if (match) {
			return { token: match[1] };
		}

		// return { error: 'Bad Authorization header format. Format is "Authorization: Bearer <token>"' }
	}

	// Tries the "authentication" cookie
	if (context.cookies.get('authentication')) {
		return { token: context.cookies.get('authentication') };
	}

	// No JWT token was found
	return { error: 'JWT token not found: no "Authorization: Bearer {token}" HTTP header and no "authentication" cookie.' };
}

// Generates and signs a JWT token
function issue_jwt_token(_ref4) {
	var payload = _ref4.payload;
	var keys = _ref4.keys;
	var user_id = _ref4.user_id;
	var jwt_id = _ref4.jwt_id;

	if (arguments.length !== 1) {
		throw new Error("`jwt` function must take a single argument: an object { payload, keys, user_id, jwt_id }");
	}

	if (!keys) {
		throw new Error('JWT encryption "keys" weren\'t supplied');
	}

	var token = _jsonwebtoken2.default.sign(payload, keys[0], {
		subject: user_id,
		jwtid: jwt_id
	});

	return token;
}
//# sourceMappingURL=authentication.js.map