'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

exports.default = web_service;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _http = require('http');

var _http2 = _interopRequireDefault(_http);

var _https = require('https');

var _https2 = _interopRequireDefault(_https);

var _koa = require('koa');

var _koa2 = _interopRequireDefault(_koa);

var _koaBodyparser = require('koa-bodyparser');

var _koaBodyparser2 = _interopRequireDefault(_koaBodyparser);

var _koaMount = require('koa-mount');

var _koaMount2 = _interopRequireDefault(_koaMount);

var _koaBunyan = require('koa-bunyan');

var _koaBunyan2 = _interopRequireDefault(_koaBunyan);

var _koaCompress = require('koa-compress');

var _koaCompress2 = _interopRequireDefault(_koaCompress);

var _koaStatic = require('koa-static');

var _koaStatic2 = _interopRequireDefault(_koaStatic);

var _koaLocale = require('koa-locale');

var _koaLocale2 = _interopRequireDefault(_koaLocale);

var _errors = require('./errors');

var _errors2 = _interopRequireDefault(_errors);

var _promisify = require('./promisify');

var _promisify2 = _interopRequireDefault(_promisify);

var _helpers = require('./helpers');

var _errorHandler = require('./middleware/error handler');

var _errorHandler2 = _interopRequireDefault(_errorHandler);

var _authentication = require('./middleware/authentication');

var _authentication2 = _interopRequireDefault(_authentication);

var _proxy = require('./middleware/proxy');

var _proxy2 = _interopRequireDefault(_proxy);

var _fileUpload = require('./middleware/file upload');

var _fileUpload2 = _interopRequireDefault(_fileUpload);

var _acl = require('./middleware/acl');

var _acl2 = _interopRequireDefault(_acl);

var _session = require('./middleware/session');

var _session2 = _interopRequireDefault(_session);

var _routing2 = require('./middleware/routing');

var _routing3 = _interopRequireDefault(_routing2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Sets up a Web Server instance (based on Koa)
//
// options:
//
// compress            - enables tar/gz compression of Http response data
//
// detect_locale       - extracts locale from Http Request headers 
//                       and places it into ctx.locale
//
// session             - tracks user session (ctx.session)
//
// authentication      - uses a JWT token as a means of user authentication
//                       (should be a function transforming token payload into user info)
//
// parse_body          - parse Http Post requests body (default: false; true when using routing)
//
// routing             - enables Rest Http routing
//                       (usage: web.get('/path', parameters => return 'Echo'))
//
// log                 - bunyan log instance
//
// csrf                - enables protection against Cross Site Request Forgery attacks
//                       (pending)
//
// returns an object with properties:
//
//   shut_down()   - gracefully shuts down the server (pending)
//
//   connections() - returns currently open connections count (not tested)
//
//   errors        - a set of common Http errors
//
//     Unauthorized
//     Access_denied
//     Not_found
//     Input_missing
//
//   file_upload() - enables file upload functionality
//
//     parameters:
//
//       path           - the URL path to mount this middleware at (defaults to /)
//
//       upload_folder  - where to write the files
//
//       multiple_files - set this flag to true in case of multiple file upload
//
//   serve_static_files() - enables serving static files
//
//     parameters:
//
//       url_path        - the URL path to mount this middleware at
//
//       filesystem_path - the corresponding filesystem path where the static files reside
//
//   listen()             - starts listening for requests
//
//     parameters:
//
//       port - the TCP port to listen on
//       host - the TCP host to listen on (defaults to 0.0.0.0)
//
//     returns: a Promise
//
//   mount()             - mounts a middleware at a path
//
//     parameters:
//
//       path       - the URL path to mount the middleware at
//       middleware - the middleware to mount
//
//   use()               - standard Koa .use() method
//
//   proxy()             - proxies all requests for this path to another web server
//
//     parameters:
//
//       path        - the URL path to mount the requests for
//       destination - where to proxy these requests to
//
function web_service() {
	var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

	// In development mode errors are printed as HTML
	var development = process.env.NODE_ENV !== 'production';

	// this object will be returned
	var result = {};

	// instantiate a Koa web application
	var web = new _koa2.default();

	// Trust `X-Forwarded-For` HTTP header
	// https://en.wikipedia.org/wiki/X-Forwarded-For
	web.proxy = true;

	if (options.compress) {
		// хз, нужно ли сжатие в node.js: мб лучше поставить впереди nginx'ы, 
		// и ими сжимать, чтобы не нагружать процесс node.js
		web.use((0, _koaCompress2.default)());
	}

	var log = options.log || {
		debug: console.info.bind(console),
		info: console.info.bind(console),
		warn: console.warn.bind(console),
		error: console.error.bind(console)
	};

	// handle errors
	web.use((0, _errorHandler2.default)({ development: development, log: log, html: options.error_html }));

	// If an Access Control List is set,
	// then allow only IPs from the list of subnets
	if (options.access_list) {
		web.use((0, _acl2.default)(options.access_list));
	}

	result.log = log;

	if (options.debug) {
		web.use((0, _koaBunyan2.default)(log, {
			// which level you want to use for logging.
			// default is info
			level: 'debug',
			// this is optional. Here you can provide request time in ms,
			// and all requests longer than specified time will have level 'warn'
			timeLimit: 100
		}));
	}

	if (options.detect_locale) {
		// get locale from Http request
		// (the second parameter is the Http Get parameter name)
		(0, _koaLocale2.default)(web, 'locale');

		// usage:
		//
		// .use(async function(ctx)
		// {
		// 	const preferred_locale = ctx.getLocaleFromQuery() || ctx.getLocaleFromCookie() || ctx.getLocaleFromHeader() || 'en'
		// })
	}

	web.keys = options.keys;

	if (options.authentication) {
		web.use((0, _authentication2.default)({
			authentication: options.authentication,
			keys: options.keys,
			validate_token: options.validate_token
		}));
	}

	if (options.session) {
		web.use((0, _session2.default)(options.redis));
	}

	if (options.parse_body !== false && options.routing === true) {
		options.parse_body = true;
	}

	if (options.parse_body) {
		// Set up http post request handling.
		// Usage: ctx.request.body
		web.use((0, _koaBodyparser2.default)({ formLimit: '100mb' }));
	}

	if (options.csrf) {
		// Cross Site Request Forgery protection
		//
		// также: в api client'е при любом запросе выставлять заголовок X-Csrf-Token = csrf token cookie.
		//
		// // Cross Site Request Forgery token check
		// web.use(function* (next)
		// {
		// 	// on login:
		// 	import crypto from 'crypto'
		// 	const hmac = crypto.createHmac('sha1', configuration.session_secret_keys.first())
		// 	hmac.update(ctx.session)
		// 	ctx.cookies.set('csrf-token', hmac.digest('hex'))
		//
		// 	// else, if logged in
		// 	if (ctx.get('X-Csrf-Token') !== ctx.cookies.get('csrf-token'))
		// 	{
		// 			throw new Errors.Access_denied(`Cross Site Request Forgery token mismatch. Expected "csrf-token" cookie value ${ctx.cookies.get('csrf-token')} to equal "X-Csrf-Token" header value ${ctx.get('X-Csrf-Token')}`)
		// 	}
		// })
	}

	if (options.routing) {
		var _routing = (0, _routing3.default)({
			keys: options.keys,
			routing: options.routing,
			parse_body: options.parse_body
		});

		var extensions = _routing.extensions;
		var middleware = _routing.middleware;
		var _iteratorNormalCompletion = true;
		var _didIteratorError = false;
		var _iteratorError = undefined;

		try {

			for (var _iterator = (0, _getIterator3.default)((0, _keys2.default)(extensions)), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
				var key = _step.value;

				result[key] = extensions[key];
			}
		} catch (err) {
			_didIteratorError = true;
			_iteratorError = err;
		} finally {
			try {
				if (!_iteratorNormalCompletion && _iterator.return) {
					_iterator.return();
				}
			} finally {
				if (_didIteratorError) {
					throw _iteratorError;
				}
			}
		}

		var _iteratorNormalCompletion2 = true;
		var _didIteratorError2 = false;
		var _iteratorError2 = undefined;

		try {
			for (var _iterator2 = (0, _getIterator3.default)(middleware), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
				var use = _step2.value;

				web.use(use);
			}
		} catch (err) {
			_didIteratorError2 = true;
			_iteratorError2 = err;
		} finally {
			try {
				if (!_iteratorNormalCompletion2 && _iterator2.return) {
					_iterator2.return();
				}
			} finally {
				if (_didIteratorError2) {
					throw _iteratorError2;
				}
			}
		}
	}

	// active Http proxy servers
	var proxies = [];

	// server shutting down flag
	var shut_down = false;

	// in case of maintenance
	web.use(function () {
		var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(ctx, next) {
			return _regenerator2.default.wrap(function _callee$(_context) {
				while (1) {
					switch (_context.prev = _context.next) {
						case 0:
							if (!shut_down) {
								_context.next = 5;
								break;
							}

							ctx.status = 503;
							ctx.message = 'The server is shutting down for maintenance';
							_context.next = 7;
							break;

						case 5:
							_context.next = 7;
							return next();

						case 7:
						case 'end':
							return _context.stop();
					}
				}
			}, _callee, this);
		}));

		return function (_x2, _x3) {
			return _ref.apply(this, arguments);
		};
	}());

	result.shut_down = function () {
		shut_down = true;

		// pending promises
		var pending = [];

		// shut down http proxy
		var _iteratorNormalCompletion3 = true;
		var _didIteratorError3 = false;
		var _iteratorError3 = undefined;

		try {
			for (var _iterator3 = (0, _getIterator3.default)(proxies), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
				var proxy = _step3.value;

				pending.push((0, _promisify2.default)(proxy.close, proxy)());
			}

			// Stops the server from accepting new connections and keeps existing connections. 
			//
			// The optional callback will be called once the 'close' event occurs. 
			// Unlike that event, it will be called with an Error as its only argument 
			// if the server was not open when it was closed.
			//
		} catch (err) {
			_didIteratorError3 = true;
			_iteratorError3 = err;
		} finally {
			try {
				if (!_iteratorNormalCompletion3 && _iterator3.return) {
					_iterator3.return();
				}
			} finally {
				if (_didIteratorError3) {
					throw _iteratorError3;
				}
			}
		}

		pending.push((0, _promisify2.default)(web.close, web)());

		return _promise2.default.all(pending);
	};

	result.connections = function () {
		// http_server.getConnections()
		return (0, _promisify2.default)(web.getConnections, web)();
	};

	// can handle file uploads
	result.file_upload = function () {
		if (options.parse_body) {
			throw new Error('.file_upload() was enabled but also "parse_body" wasn\'t set to false, therefore Http POST request bodies are parsed which creates a conflict. Set "parse_body" parameter to false.');
		}

		web.use(_fileUpload2.default.apply(this, arguments));
	};

	// Shorter alias
	result.upload = result.file_upload;

	// can serve static files
	result.serve_static_files = function (url_path, filesystem_path) {
		// https://github.com/koajs/static
		web.use((0, _koaMount2.default)(url_path, (0, _koaStatic2.default)(filesystem_path, {
			maxAge: 365 * 24 * 60 * 60 // 1 year
		})));
	};

	// Shorter alias
	result.files = result.serve_static_files;

	// mounts middleware at path
	result.mount = function (path, handler) {
		web.use((0, _koaMount2.default)(path, handler));
	};

	// exposes Koa .use() function for custom middleware
	result.use = web.use.bind(web);

	// Proxies all URLs starting with 'from_path' to another server
	// (make sure you proxy only to your own servers
	//  so that you don't leak cookies or JWT tokens to the 3rd party)
	result.proxy = function (from_path, to) {
		var _proxier = (0, _proxy2.default)(from_path, to);

		var proxy = _proxier.proxy;
		var middleware = _proxier.middleware;

		proxies.push(proxy);
		web.use(middleware);
	};

	// runs http server
	result.listen = function (port, host) {
		if ((0, _helpers.is_object)(port)) {
			host = port.host;
			port = port.port;
		}

		host = host || '0.0.0.0';

		return new _promise2.default(function (resolve, reject) {
			// the last route - throws not found error
			web.use(function () {
				var _ref2 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2(ctx) {
					return _regenerator2.default.wrap(function _callee2$(_context2) {
						while (1) {
							switch (_context2.prev = _context2.next) {
								case 0:
									// throw new Method_not_found()
									ctx.status = 404;
									ctx.message = 'The requested resource not found: ' + ctx.method + ' ' + ctx.url;

									// Reduces noise in the `log` in case of errors
									// (browsers query '/favicon.ico' automatically)
									if (!(0, _helpers.ends_with)(ctx.path, '/favicon.ico')) {
										log.error(ctx.message, 'Web server error: Not found');
									}

								case 3:
								case 'end':
									return _context2.stop();
							}
						}
					}, _callee2, this);
				}));

				return function (_x4) {
					return _ref2.apply(this, arguments);
				};
			}());

			// http server
			var http_web_server = _http2.default.createServer();

			// // enable Koa for handling http requests
			// http_web_server.on('request', web.callback())

			// copy-pasted from 
			// https://github.com/koajs/koala/blob/master/lib/app.js
			//
			// "Expect: 100-continue" is something related to http request body parsing
			// http://crypto.pp.ua/2011/02/mexanizm-expectcontinue/
			//
			var koa_callback = web.callback();
			http_web_server.on('request', koa_callback);
			http_web_server.on('checkContinue', function (request, response) {
				// requests with `Expect: 100-continue`
				request.checkContinue = true;
				koa_callback(request, response);
			});

			http_web_server.listen(port, host, function (error) {
				if (error) {
					return reject(error);
				}

				resolve(http_web_server);
			});
			// .on('connection', () => connections++)
			// .on('close', () => connections--)
		});
	};

	// done
	return result;
}
module.exports = exports['default'];
//# sourceMappingURL=web service.js.map